@@ -34,8 +34,8 @@
 #'
 #' # Use dplyr::mutate(dplyr::across(...)) to assign an explicit type
 #' tribble(
-#'   ~ a,          ~ b,          ~ c,
-#'     1, "2000-01-01",        "1.5"
+#'   ~a, ~b, ~c,
+#'   1, "2000-01-01", "1.5"
 #' ) %>%
 #'   dplyr::mutate(
 #'     dplyr::across(a, as.integer),
@@ -75,19 +75,19 @@ frame_matrix <- function(...) {
   turn_frame_data_into_frame_matrix(data$frame_names, data$frame_rest)
 }
 
-extract_frame_data_from_dots <- function(...) {
+extract_frame_data_from_dots <- function(..., .call = caller_env()) {
   dots <- list2(...)
 
   # Extract the names.
-  frame_names <- extract_frame_names_from_dots(dots)
+  frame_names <- extract_frame_names_from_dots(dots, .call)
 
   # Extract the data
   if (length(frame_names) == 0 && length(dots) != 0) {
-    cnd_signal(error_tribble_needs_columns())
+    abort_tribble_needs_columns(.call)
   }
   frame_rest <- dots[-seq_along(frame_names)]
   if (!is.null(names(frame_rest))) {
-    cnd_signal(error_tribble_named_after_tilde())
+    abort_tribble_named_after_tilde(.call)
   }
   if (length(frame_rest) == 0L) {
     # Can't decide on type in absence of data -- use logical which is
@@ -95,12 +95,12 @@ extract_frame_data_from_dots <- function(...) {
     frame_rest <- unspecified()
   }
 
-  validate_rectangular_shape(frame_names, frame_rest)
+  validate_rectangular_shape(frame_names, frame_rest, .call)
 
   list(frame_names = frame_names, frame_rest = frame_rest)
 }
 
-extract_frame_names_from_dots <- function(dots) {
+extract_frame_names_from_dots <- function(dots, call = caller_env()) {
   frame_names <- character()
 
   for (i in seq_along(dots)) {
@@ -114,12 +114,12 @@ extract_frame_names_from_dots <- function(dots) {
     }
 
     if (length(el) != 2) {
-      cnd_signal(error_tribble_lhs_column_syntax(el[[2]]))
+      abort_tribble_lhs_column_syntax(el[[2]], call)
     }
 
     candidate <- el[[2]]
     if (!(is.symbol(candidate) || is.character(candidate))) {
-      cnd_signal(error_tribble_rhs_column_syntax(candidate))
+      abort_tribble_rhs_column_syntax(candidate)
     }
 
     frame_names <- c(frame_names, as.character(candidate))
@@ -128,21 +128,18 @@ extract_frame_names_from_dots <- function(dots) {
   frame_names
 }
 
-validate_rectangular_shape <- function(frame_names, frame_rest) {
+validate_rectangular_shape <- function(frame_names, frame_rest, call = caller_env()) {
   if (length(frame_names) == 0 && length(frame_rest) == 0) return()
 
   # Figure out the associated number of rows and number of columns,
   # and validate that the supplied formula produces a rectangular
   # structure.
   if (length(frame_rest) %% length(frame_names) != 0) {
-    cnd_signal(error_tribble_non_rectangular(
-      length(frame_names),
-      length(frame_rest)
-    ))
+    abort_tribble_non_rectangular(length(frame_names), length(frame_rest), call)
   }
 }
 
-turn_frame_data_into_tibble <- function(names, rest) {
+turn_frame_data_into_tibble <- function(names, rest, call = caller_env()) {
   if (is_empty(names)) return(new_tibble(list(), nrow = 0))
 
   nrow <- length(rest) / length(names)
@@ -150,12 +147,12 @@ turn_frame_data_into_tibble <- function(names, rest) {
   dimnames(rest) <- list(names, NULL)
 
   frame_mat <- t(rest)
-  frame_col <- turn_matrix_into_column_list(frame_mat)
+  frame_col <- turn_matrix_into_column_list(frame_mat, call)
 
   new_tibble(frame_col, nrow = nrow)
 }
 
-turn_matrix_into_column_list <- function(frame_mat) {
+turn_matrix_into_column_list <- function(frame_mat, call) {
   frame_col <- vector("list", length = ncol(frame_mat))
   names(frame_col) <- colnames(frame_mat)
 
@@ -166,7 +163,8 @@ turn_matrix_into_column_list <- function(frame_mat) {
     if (inherits(col, "list") && !some(col, needs_list_col)) {
       subclass_tribble_c_errors(
         names(frame_col)[[i]],
-        col <- vec_c(!!! unname(col))
+        col <- vec_c(!!!unname(col)),
+        call
       )
     }
 
@@ -175,10 +173,10 @@ turn_matrix_into_column_list <- function(frame_mat) {
   return(frame_col)
 }
 
-turn_frame_data_into_frame_matrix <- function(names, rest) {
+turn_frame_data_into_frame_matrix <- function(names, rest, call = caller_env()) {
   list_cols <- which(map_lgl(rest, needs_list_col))
   if (has_length(list_cols)) {
-    cnd_signal(error_frame_matrix_list(list_cols))
+    abort_frame_matrix_list(list_cols, call)
   }
 
   frame_ncol <- length(names)
@@ -188,58 +186,59 @@ turn_frame_data_into_frame_matrix <- function(names, rest) {
   frame_mat
 }
 
-subclass_tribble_c_errors <- function(name, code) {
+subclass_tribble_c_errors <- function(name, code, call) {
   withCallingHandlers(
     code,
-
     vctrs_error = function(cnd) {
-      cnd_signal(error_tribble_c(name, cnd))
+      abort_tribble_c(name, cnd, call)
     }
   )
 }
 
 # Errors ------------------------------------------------------------------
 
-error_tribble_needs_columns <- function() {
-  tibble_error("Must specify at least one column using the `~name` syntax.")
+abort_tribble_needs_columns <- function(call = caller_env()) {
+  tibble_abort(call = call, "Must specify at least one column using the `~name` syntax.")
 }
 
-error_tribble_named_after_tilde <- function() {
-  tibble_error("When using the `~name` syntax, subsequent values must not have names.")
+abort_tribble_named_after_tilde <- function(call = caller_env()) {
+  tibble_abort(call = call, "When using the `~name` syntax, subsequent values must not have names.")
 }
 
-error_tribble_lhs_column_syntax <- function(lhs) {
-  tibble_error(bullets(
+abort_tribble_lhs_column_syntax <- function(lhs, call = caller_env()) {
+  tibble_abort(call = call, problems(
     "All column specifications must use the `~name` syntax.",
     paste0("Found ", expr_label(lhs), " on the left-hand side of `~`.")
   ))
 }
 
-error_tribble_rhs_column_syntax <- function(rhs) {
-  tibble_error(bullets(
+abort_tribble_rhs_column_syntax <- function(rhs, call = caller_env()) {
+  tibble_abort(call = call, problems(
     'All column specifications must use the `~name` or `~"name"` syntax.',
     paste0("Found ", expr_label(rhs), " on the right-hand side of `~`.")
   ))
 }
 
-error_tribble_non_rectangular <- function(cols, cells) {
-  tibble_error(bullets(
+abort_tribble_non_rectangular <- function(cols, cells, call = caller_env()) {
+  tibble_abort(call = call, bullets(
     "Data must be rectangular:",
     paste0("Found ", cols, " columns."),
     paste0("Found ", cells, " cells."),
-    i = paste0(cells, " is not an integer multiple of ", cols, ".")
+    info = paste0(cells, " is not an integer multiple of ", cols, ".")
   ))
 }
 
-error_frame_matrix_list <- function(pos) {
-  tibble_error(bullets(
+abort_frame_matrix_list <- function(pos, call = caller_env()) {
+  tibble_abort(call = call, problems(
     "All values must be atomic:",
     pluralise_commas("Found list-valued element(s) at position(s) ", pos, ".")
   ))
 }
 
-error_tribble_c <- function(name, cnd) {
-  cnd$message <- paste0("Can't create column ", tick(name), ": ", cnd_header(cnd))
-  cnd$class <- c(tibble_error_class("tribble_c"), class(cnd))
-  cnd
+abort_tribble_c <- function(name, cnd, call) {
+  tibble_abort(
+    paste0("Can't create column ", tick(name)),
+    parent = cnd,
+    call = call
+  )
 }
