@@ -22,10 +22,15 @@ test_that("pluralise_msg works correctly", {
 
 test_that("output test", {
   expect_snapshot({
+    "# Problems"
+    writeLines(format_error_bullets(problems("header", c("item 1", "item 2"))))
+    writeLines(format_error_bullets(problems("header", LETTERS)))
+    writeLines(format_error_bullets(problems("header", as.character(1:6))))
+
     "# Bullets"
-    bullets("header", c("item 1", "item 2"))
-    bullets("header", LETTERS)
-    bullets("header", 1:6)
+    writeLines(format_error_bullets(bullets("header", c("item 1", "item 2"))))
+    writeLines(format_error_bullets(bullets("header", LETTERS)))
+    writeLines(format_error_bullets(bullets("header", as.character(1:6))))
 
     "# Commas"
     commas("1")
